/*	mkhdr.c
**	Make a header file out of a list of modules
**	syntax:	mkhdr [-c -s] <hdrfile> {<module> ...}
*/

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

//#define DEBUG

typedef enum
	{
	        mode_NONE,
		mode_C,
		mode_ASM,
	} mode_t;

#define TMP_SIZ		64

static int ToLower( char *dest, const char *src, int max )
{
        int chars = 0;

        while ( (*src != '\0') && (chars < max) )
        {
               	*dest++ = (char)tolower( *src++ );
               	chars++;
        }

	*dest = '\0';
        return chars;
}

int main (int argc, const char **argv)
{
	int status = 0;
	mode_t mode = mode_NONE;
	FILE *hdr = 0;

	fprintf( stderr, "mkhdr " __DATE__ " Tony van der Hoff\n\n" );

	if ( argc < 4 )
	{
		fprintf( stderr,
			"Usage:\tmkhdr [options] hdrfile module1 module2 ... moduleN\n\n"
			"Options:\t-c:\tgenerate c-style header file\n"
			"\t\t-s:\tgenerate objasm-style header file\n"
			);
		status = -1;
	}

	if ( status == 0 )
	{
#ifdef DEBUG
		fprintf( stderr,
			"Option: %s\n"
			"Output: %s\n"
			, argv[1]
			, argv[2]
			);
		for ( int i = 3; i < argc; ++i )
		{
		        fprintf( stderr, "%s ", argv[i] );
		}
		fputs( "\n",stderr );
#endif
		if ( strcmp( argv[1], "-s" ) == 0 )
			mode = mode_ASM;
		else if ( strcmp( argv[1], "-c" ) == 0 )
			mode = mode_C;

		if ( mode == mode_NONE )
		{
		        fprintf( stderr,
		        	"Error: invalid or no translation mode defined\n"
		        	);
		        status = -1;
		}
	}

	if (status == 0)
	{
		hdr = fopen( argv[2], "w" );
		if ( hdr == 0 )
		{
		        fprintf( stderr,
		        	"Error: File %s failed to open\n",
		        	argv[2]
		        	);
		        status = -1;
		}
		else
		{
		}
        }

	if (status == 0)
	{
	        time_t tim = time( 0 );

	        switch ( mode )
	        {
	        case mode_C:
			fprintf( hdr,
		      		"/* %s - Generated by mkhdr [" __DATE__ "] on %s */\n",
		       		argv[2], ctime( &tim )
		       		);
		       		break;

		case mode_ASM:
			fprintf( hdr,
		      		";%s - Generated by mkhdr [" __DATE__ "] on %s\n",
		       		argv[2], ctime( &tim )
		       		);
		       	break;
		}

	        for ( int i = 3; i < argc; ++i )
	        {
	                char temp[ TMP_SIZ ];

	                switch ( mode )
	                {
	                case mode_C:
	                	ToLower( temp, argv[i], TMP_SIZ );
	                	fprintf( hdr,
	                		"#include \"oslib/%s.h\"\n",
	                		temp
	                		);
	                	break;

	                case mode_ASM:
	                	fprintf( hdr,
	                		//"\tGET\toslib.Hdr.%s\n",
	                		"\tGET\toslib/%s.Hdr\n",
	                		argv[i]
	                		);
	                	break;
	                }	/* end switch */
	        }	/* end for */

	        switch ( mode )
	        {
	        	case mode_ASM:
	                	fprintf( hdr,
	                		"\tEND\n"
	                		);
	                	break;
	        }
        }	/* status == 0 */

        if ( hdr )
        	fclose( hdr );

   	return status;
}
